﻿
using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace NB_IOT_UDP_HP_SCOKET.WWKJUtil
{
    class DBHelper
    {
        //连接字符串
        private static readonly string connStr = ConfigurationManager.AppSettings["db"];
        

        public static int ExecuteNonQuery(string sql, CommandType cmdType, params SqlParameter[] pms)
        {

            using (SqlConnection con = new SqlConnection(connStr))
            {
                using (SqlCommand cmd = new SqlCommand(sql, con))
                {
                    //设置当前执行的是存储过程还是带参数的Sql语句
                    cmd.CommandType = cmdType;
                    if (pms != null)
                    {
                        cmd.Parameters.AddRange(pms);
                    }

                    try
                    {
                        con.Open();
                        return cmd.ExecuteNonQuery();
                    }
                    catch
                    {
                        con.Close();
                        con.Dispose();
                        throw;
                    }
                }
            }
        }
        

        public static int ExecuteNoQuery(string sql, params SqlParameter[] parameters)
        {
            using (SqlConnection conn = new SqlConnection(connStr))
            {
               
                using (SqlCommand cmd = conn.CreateCommand())
                {
                    cmd.CommandText = sql;

                    foreach (SqlParameter parameter in parameters)
                    {
                        cmd.Parameters.Add(parameter);
                    }
                    conn.Open();
                    return cmd.ExecuteNonQuery();
                }

            }
        }

        //2.执行返回单个值的方法
        public static object ExecuteScalar(string sql, CommandType cmdType, params SqlParameter[] pms)
        {
            
            ////拼接链接字符串
            //string connStrNew = "data source=" + SqlServerIP + ";initial catalog=" + dbName + ";user id=" + userId + ";password=" + pwd + ";";
            using (SqlConnection con = new SqlConnection(connStr))
            {
                using (SqlCommand cmd = new SqlCommand(sql, con))
                {
                    cmd.Parameters.Clear();
                    //设置当前执行的是存储过程还是带参数的Sql语句
                    cmd.CommandType = cmdType;
                    if (pms != null)
                    {
                        cmd.Parameters.AddRange(pms);
                    }

                    con.Open();
                    return cmd.ExecuteScalar();
                }
            }
        }


        //3.返回SqlDataReader的方法
        public static SqlDataReader ExecuteReader(string sql, CommandType cmdType, params SqlParameter[] pms)
        {
            SqlConnection con = new SqlConnection(connStr);

            using (SqlCommand cmd = new SqlCommand(sql, con))
            {
                cmd.CommandType = cmdType;
                if (pms != null)
                {
                    cmd.Parameters.AddRange(pms);
                }
                try
                {
                    
                    SqlDataReader dr = cmd.ExecuteReader(CommandBehavior.CloseConnection); //修改 刘学铸 by 2017-06-02
                    cmd.Parameters.Clear();
                    return dr;//cmd.ExecuteReader(CommandBehavior.CloseConnection);
                }
                catch
                {
                    con.Close();
                    con.Dispose();
                    throw;
                }
            }

        }

        //4.返回DataTable
        public static DataTable ExecuteDataTable(string sql, CommandType cmdType, params SqlParameter[] pms)
        {
            DataTable dt = new DataTable();
            using (SqlDataAdapter adapter = new SqlDataAdapter(sql, connStr))
            {
                adapter.SelectCommand.CommandTimeout = 600;
                adapter.SelectCommand.CommandType = cmdType;
                if (pms != null)
                {
                    adapter.SelectCommand.Parameters.AddRange(pms);
                }
                adapter.Fill(dt);
                return dt;
            }
        }


        /// <summary>
        /// 执行sql语句无参数
        /// </summary>
        public static int ExecuteNoParams(string sql, CommandType cmdType)
        {
            using (SqlConnection conn = new SqlConnection(connStr))
            {
                
                using (SqlCommand cmd = new SqlCommand(sql, conn))
                {
                    cmd.CommandType = cmdType;
                    try
                    {
                        conn.Open();
                        return cmd.ExecuteNonQuery();
                    }
                    catch (Exception)
                    {
                        conn.Close();
                        throw;
                    }
                }

            }



        }

        /// <summary>
        /// 带事务的存储过程 
        /// </summary>
        /// <param name="procName">存储过程名称</param>
        /// <param name="parameters">存储过程参数-不能为空</param>
        /// <returns>0:执行成功 异常抛出</returns>
        public static int ExecStoredProcedure(string procName, params SqlParameter[] parameters)
        {
            int rtn = 0;
            using (SqlConnection conn = new SqlConnection(connStr))
            {
                
                using (SqlCommand cmd = conn.CreateCommand())
                {
                    SqlTransaction st = conn.BeginTransaction();
                    cmd.Transaction = st;
                    try
                    {
                        // cmd.SelectCommand.CommandTimeout = 600;
                        cmd.CommandTimeout = 600;
                        cmd.CommandText = procName;
                        cmd.CommandType = CommandType.StoredProcedure;
                        if (parameters != null)
                        {
                            cmd.Parameters.AddRange(parameters);
                        }
                        conn.Open();
                        rtn = cmd.ExecuteNonQuery();
                        st.Commit();
                        cmd.Parameters.Clear();//修改 刘学铸 at 2017-07-13
                        return rtn;
                    }
                    catch (SqlException sqlex)
                    {
                        st.Rollback();
                        throw sqlex;
                    }
                }
            }
        }


        /// <summary>
        /// 事务的sql
        /// </summary>
        /// <param name="sqlStr"></param>
        /// <param name="TranName"></param>
        /// <returns></returns>
        public static bool ExecuteTrasaction(string sqlStr, string TranName)
        {
            bool result = true;
            SqlTransaction sqlTransaction = null;
            SqlConnection sqlConnection = new SqlConnection(connStr);
            try
            {
                
                sqlTransaction = sqlConnection.BeginTransaction(TranName);
                SqlCommand sqlCommand = new SqlCommand(sqlStr, sqlConnection, sqlTransaction);
                sqlConnection.Open();
                int num = sqlCommand.ExecuteNonQuery();
                if (num > 0)
                {
                    sqlTransaction.Commit();
                    sqlConnection.Close();
                    result = true;
                }
                else
                {
                    sqlTransaction.Rollback();
                    sqlConnection.Close();
                    result = false;
                }
            }
            catch
            {
                sqlTransaction.Rollback();
                sqlConnection.Close();
                result = false;
            }
            return result;
        }

        /// <summary>
        /// 返回DataTable
        /// </summary>
        /// <param name="sql"></param>
        /// <returns></returns>
        public static DataTable Fill(string sql)
        {
            return ExecuteDataTable(sql, CommandType.Text, null);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sql"></param>
        /// <returns></returns>
        public static int Query(string sql)
        {
            return  ExecuteNonQuery(sql, CommandType.Text, null);
        }

        public static int ScalarToInt(string sql)
        {
            Object obj = ExecuteScalar(sql, CommandType.Text, null);
            if (obj == null)
            {
                return 0;
            }
            else
            {
                return (int)obj;
            }
        }

        #region 返回 sqlDataReader
        public static SqlDataReader ReturnSqlDataReader(String sql)
        {
            SqlConnection conn = new SqlConnection(connStr);
            SqlCommand cmd = new SqlCommand(sql, conn);

            conn.Open();

            try
            {
                SqlDataReader reader = cmd.ExecuteReader(CommandBehavior.CloseConnection);
                return reader;
            }
            catch (Exception ex)
            {
                conn.Close();
                throw ex;
            }


        }
        #endregion
    }
}
